//
//  ViewController.swift
//  DistanceSensor
//
//  Created by Phidgets on 2018-01-29.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {
    let ch = DistanceSensor()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var distanceLabel: UILabel!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var changeTriggerSlider: UISlider!
    @IBOutlet var changeTriggerLabel: UILabel!
    @IBOutlet var sonarStackview: UIStackView!
    @IBOutlet var sonarDistanceCollection: [UILabel]!
    @IBOutlet var sonarAmplitudeCollection: [UILabel]!
    @IBOutlet var quietModeButton: UIButton!
    @IBOutlet var stackView: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.distanceChange.addHandler(distancechange_handler)
            let _ = ch.sonarReflectionsUpdate.addHandler(sonarupdate_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }

    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! DistanceSensor
            
            let deviceID = try attachedDevice.getDeviceID()
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            let minDistanceChangeTrigger = try Float(attachedDevice.getMinDistanceChangeTrigger())
            let maxDistanceChangeTrigger = try Float(attachedDevice.getMaxDistanceChangeTrigger())
            let distanceChangeTrigger = try Float(attachedDevice.getDistanceChangeTrigger())
            
            
            switch(deviceID){
            case DeviceID.PN_DST1200:
                let isQuietModeEnabled = try attachedDevice.getSonarQuietMode()
                let buttonTitle = isQuietModeEnabled ? "Disable Quiet Mode" : "Enable Quiet Mode"
                DispatchQueue.main.async {
                    self.quietModeButton.setTitle(buttonTitle, for: UIControlState.normal)
                    self.sonarStackview.isHidden = false
                }
                break
            default:
                DispatchQueue.main.async {
                    self.sonarStackview.isHidden = true
                }
                break
            }
            
                DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format: "%d ms", UInt32(dataInterval))
                //if min=max interval can not be configured
                if(minDataInterval == maxDataInterval){
                    self.dataIntervalSlider.isEnabled = false
                }
                self.changeTriggerSlider.minimumValue = minDistanceChangeTrigger
                self.changeTriggerSlider.maximumValue = maxDistanceChangeTrigger
                self.changeTriggerSlider.value = distanceChangeTrigger
                self.changeTriggerLabel.text = String(format: "%d mm", distanceChangeTrigger)
            }
            
            
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func distancechange_handler(sender: DistanceSensor, distance: UInt32){
        DispatchQueue.main.async {
            self.distanceLabel.text = "\(distance)"
        }
    }
    
    func sonarupdate_handler(sender: DistanceSensor, data: (distances: [UInt32], amplitudes: [UInt32], count: UInt32)){
        DispatchQueue.main.async {
            
            for index in 0...7{
                if(index < data.count){
                    self.sonarDistanceCollection[index].text = "[\(index)] \(data.distances[index])"
                    self.sonarAmplitudeCollection[index].text = "[\(index)] \(data.amplitudes[index])"
                }
                else{
                    self.sonarDistanceCollection[index].text = "[\(index)]"
                    self.sonarAmplitudeCollection[index].text = "[\(index)]"
                }
            }
        }
    }
    
    //MARK: GUI controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            DispatchQueue.main.async {
                self.dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setChangeTrigger(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDistanceChangeTrigger(UInt32(slider.value))
            DispatchQueue.main.async {
                self.changeTriggerLabel.text = String(format:"%d mm", UInt32(slider.value))
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setQuietMode(_ sender: Any) {
        let button = sender as! UIButton
        do{
            let quietModeEnabled = try ch.getSonarQuietMode()
            let buttonTitle = quietModeEnabled ? "Enable Quiet Mode" : "Disable Quiet Mode"
            try ch.setSonarQuietMode(!quietModeEnabled)
            DispatchQueue.main.async {
                    button.setTitle(buttonTitle, for: UIControlState.normal)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }


}


